import { RefTyped, unwrap } from "../utils";
import { computed, Ref } from "../../src/api";
import { intlDateFormatExtractArguments } from "./_helper";
import { IntlNumberFormatOptions } from "./types";

export type IntlNumberFormatLocales =
  | RefTyped<string>
  | RefTyped<string[]>
  | undefined;

export type IntNumberFormatterFormat<T> = (
  value: Readonly<RefTyped<Readonly<number>>>,
  overrideOpts?: RefTyped<Intl.NumberFormatOptions>,
  overrideLocale?: RefTyped<IntlNumberFormatLocales>
) => T;

export interface NumberFormatReturn {
  formatter: Ref<Readonly<Intl.NumberFormat>>;

  format: IntNumberFormatterFormat<Ref<Readonly<string>>>;
  formatString: IntNumberFormatterFormat<string>;
}

export function useIntlNumberFormat(): NumberFormatReturn;

export function useIntlNumberFormat(
  locales: IntlNumberFormatLocales
): NumberFormatReturn;

export function useIntlNumberFormat(
  options: RefTyped<IntlNumberFormatOptions>
): NumberFormatReturn;

export function useIntlNumberFormat(
  options: RefTyped<Intl.NumberFormatOptions>
): NumberFormatReturn;

export function useIntlNumberFormat(
  locales: IntlNumberFormatLocales,
  options?: RefTyped<IntlNumberFormatOptions | undefined>
): NumberFormatReturn;

export function useIntlNumberFormat(
  locales: IntlNumberFormatLocales,
  options: RefTyped<Intl.NumberFormatOptions | undefined>
): NumberFormatReturn;

export function useIntlNumberFormat(
  localesOptions?:
    | IntlNumberFormatLocales
    | RefTyped<IntlNumberFormatOptions>
    | RefTyped<Intl.NumberFormatOptions>,
  opts?: any
) {
  const [locales, options] = intlDateFormatExtractArguments(
    localesOptions as any,
    opts
  );

  const formatter = computed(
    () =>
      new Intl.NumberFormat(
        unwrap(locales as Ref<string | string[]> | undefined),
        unwrap(options)
      )
  );

  const formatString = (
    value: RefTyped<number>,
    overrideOpts?: RefTyped<Intl.NumberFormatOptions>,
    overrideLocale?: RefTyped<IntlNumberFormatLocales>
  ) => {
    const f =
      overrideOpts || overrideLocale
        ? new Intl.NumberFormat(
            unwrap(overrideLocale as RefTyped<string>) ||
              unwrap(locales as Ref<string | string[]>),
            { ...unwrap(options), ...unwrap(overrideOpts) }
          )
        : formatter.value;

    return f.format(unwrap(value));
  };

  const format = (
    value: RefTyped<number>,
    overrideOpts?: RefTyped<Intl.NumberFormatOptions>,
    overrideLocale?: RefTyped<IntlNumberFormatLocales>
  ) => computed(() => formatString(value, overrideOpts, overrideLocale));

  return {
    format,
    formatString,

    formatter
  };
}
